#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include "hdfi.h"

#include "porting_defines.h"
#include "crisSisGetUtilities.h"
#include "utilities.h"

void writeArchiveFilesTimeStampInfo(int day1996, char *structName) {
    int overlapStructCount, i, j, structSize, dataPeriod, slotCount;
    uint32 zerothSecond, lastSecond, start, end;

    uint32 *pNonOverlapTimeStamps_0, *pNonOverlapTimeStamps;
    uint32 *pNonOverlapStopTimeStamps_0, *pNonOverlapStopTimeStamps;

    uint32 *pOverlapTimeStamps_0, *pOverlapTimeStamps;
    uint32 *pOverlapStopTimeStamps_0, *pOverlapStopTimeStamps;

    char fileName[256], dataFileSuffix[5], instrumentID;
    struct tm *pTimeStruct;

    structSize = getStructSizeFromName(structName);
    dataPeriod = getDataTypePeriodFromStructName(structName);

    pTimeStruct = getDateFromDay1996(day1996);
    instrumentID = getInstrumentIDFromStructName(structName);
    getDataFileSuffixFromStructName(dataFileSuffix, structName);

	/* read in any overlap file for this date and data type */
	/*     and get start and stop time stamps */
    overlapStructCount = getOverlapStructCount(day1996, structName);
    if (overlapStructCount > 0) {
	char *pOverlapStructs_0, *pOverlapStructs;
	int fd, bytesToRead;

	pOverlapStructs_0 = calloc(overlapStructCount, structSize);

	generateDataFilePath(fileName, pTimeStruct,
					instrumentID, dataFileSuffix);
	strcat(fileName, ".overlap");

	fd = open(fileName, O_RDONLY | O_BINARY, 0);
	bytesToRead = overlapStructCount * structSize;
	readFromFile(fd, (char *) pOverlapStructs_0, bytesToRead);
	close(fd);

	pOverlapTimeStamps_0 = calloc(overlapStructCount, sizeof(uint32));
	if (dataPeriod <= 0) {
	    pOverlapStopTimeStamps_0 =
				calloc(overlapStructCount, sizeof(uint32));
	}
	else { pOverlapStopTimeStamps_0 = NULL; }

	pOverlapTimeStamps = pOverlapTimeStamps_0;
	pOverlapStopTimeStamps = pOverlapStopTimeStamps_0;
	pOverlapStructs = pOverlapStructs_0;
	for (i = 0; i < overlapStructCount; i++) {
	    *pOverlapTimeStamps = getTimeStamp(pOverlapStructs, structName);
	    pOverlapTimeStamps++;
	    if (dataPeriod <= 0) {
		*pOverlapStopTimeStamps =
				getStopTimeStamp(pOverlapStructs, structName);
		pOverlapStopTimeStamps++;
	    }
	    pOverlapStructs += structSize;
	}

	free(pOverlapStructs_0);
    }
    else {
	pOverlapTimeStamps_0 = NULL;
	pOverlapStopTimeStamps_0 = NULL;
    }

    printf("day1996 = %d, struct name = %s, ", day1996, structName);
    printf("overlap struct count = %d\n", overlapStructCount);
    printf("%s period = %d\n", structName, dataPeriod);

    zerothSecond = getSecondsSince1996FromDay1996(day1996);
    lastSecond = getSecondsSince1996FromDay1996(day1996 + 1) - 1;
    printf("0th and last second of the day: %ld, %ld\n",
						zerothSecond, lastSecond);
    pNonOverlapTimeStamps_0 = getArchiveFilesTimeStamps(day1996, structName);
    if (dataPeriod <= 0) {
	pNonOverlapStopTimeStamps_0 =
			getArchiveFilesStopTimeStamps(day1996, structName);
    }
    else { pNonOverlapStopTimeStamps_0 = NULL; }

    if (dataPeriod > 0) {
	int daysLength;

	daysLength = (int) (lastSecond - zerothSecond + 1);
        slotCount = daysLength / dataPeriod;
        if (daysLength % dataPeriod) { slotCount++; }

        pNonOverlapTimeStamps = pNonOverlapTimeStamps_0;
        for (i = 0; i < slotCount; i++) {
	    while (*pNonOverlapTimeStamps == -1L) { pNonOverlapTimeStamps++; }

	    start = zerothSecond + (i * dataPeriod);
	    end = start + dataPeriod - 1;
	    if (end > lastSecond) { end = lastSecond; }
	    printf("%d (slot %ld - %ld): ", i, start, end);

	    if ((*pNonOverlapTimeStamps >= start) &&
					(*pNonOverlapTimeStamps <= end)) {
	        printf("data time stamp = %ld\n", *pNonOverlapTimeStamps);
	        pNonOverlapTimeStamps++;
	    }
	    else { printf("no data\n"); }

	    if (overlapStructCount > 0) {
	        pOverlapTimeStamps = pOverlapTimeStamps_0;
	        for (j = 0; j < overlapStructCount; j++) {
		    if ((*pOverlapTimeStamps >= start) &&
					(*pOverlapTimeStamps <= end)) {
		        printf("\toverlap data time stamp = %ld\n",
							*pOverlapTimeStamps);
		        pOverlapTimeStamps++;
		    }
	        }
	    }
        }
    }
    else {
	pNonOverlapTimeStamps = pNonOverlapTimeStamps_0;
	pNonOverlapStopTimeStamps = pNonOverlapStopTimeStamps_0;

	    /* print overlap structs stamped before any non-overlaps */
	pOverlapTimeStamps = pOverlapTimeStamps_0;
	pOverlapStopTimeStamps = pOverlapStopTimeStamps_0;
	for (i = 0; i < overlapStructCount; i++) {
	    if (*pOverlapTimeStamps < *pNonOverlapTimeStamps) {
		printf("\toverlap data start and stop times = %ld %ld\n",
				*pOverlapTimeStamps, *pOverlapStopTimeStamps);
		pOverlapTimeStamps++;
		pOverlapStopTimeStamps++;
	    }
	}

	    /* print non-overlap and overlap structs in start time order */
	while (*pNonOverlapTimeStamps != 0) {
	    printf("data start and stop times = %ld %ld\n",
			*pNonOverlapTimeStamps, *pNonOverlapStopTimeStamps);

	    pOverlapTimeStamps = pOverlapTimeStamps_0;
	    pOverlapStopTimeStamps = pOverlapStopTimeStamps_0;
	    for (i = 0; i < overlapStructCount; i++) {
		if ((*pOverlapTimeStamps >= *pNonOverlapTimeStamps) &&
			(*pOverlapTimeStamps <= *pNonOverlapStopTimeStamps)) {
		    printf("\toverlap data start and stop times = %ld %ld\n",
				*pOverlapTimeStamps, *pOverlapStopTimeStamps);
		    pOverlapTimeStamps++;
		    pOverlapStopTimeStamps++;
		}
	    }

	    pNonOverlapTimeStamps++;
	    pNonOverlapStopTimeStamps++;
	}

	    /* print overlap structs stamped after any non-overlaps */
	pNonOverlapStopTimeStamps--;
	pOverlapTimeStamps = pOverlapTimeStamps_0;
	pOverlapStopTimeStamps = pOverlapStopTimeStamps_0;
	for (i = 0; i < overlapStructCount; i++) {
	    if (*pOverlapTimeStamps > *pNonOverlapStopTimeStamps) {
		printf("\toverlap data start and stop times = %ld %ld\n",
			*pOverlapTimeStamps, *pOverlapStopTimeStamps);
		pOverlapTimeStamps++;
		pOverlapStopTimeStamps++;
	    }
	}
    }
		
    free(pNonOverlapTimeStamps_0);
    free(pNonOverlapStopTimeStamps_0);
    free(pOverlapTimeStamps_0);
    free(pOverlapStopTimeStamps_0);

    printf("\n\n");
}
